@@ -1,17 +1,17 @@
 package Parse::CPAN::Packages;
 use strict;
 use base qw( Class::Accessor::Chained );
- __PACKAGE__->mk_accessors(qw( filename data ));
+__PACKAGE__->mk_accessors(qw( filename data dists ));
 use CPAN::DistnameInfo;
 use Parse::CPAN::Packages::Package;
 use vars qw($VERSION);
-$VERSION = '2.12';
+$VERSION = '2.17';
 
 sub new {
   my $class    = shift;
   my $filename = shift;
 
-  my $self = {};
+  my $self = { dists => {} };
   bless $self, $class;
 
   $filename = '02packages.details.txt' if not defined $filename;
@@ -39,18 +39,22 @@ sub parse {
     $m->package($package);
     $m->version($packageversion);
 
-    my $i = CPAN::DistnameInfo->new($prefix);
+    my $dist = $self->dists->{ $prefix } ||= do {
+      my $d = Parse::CPAN::Packages::Distribution->new;
+      my $i = CPAN::DistnameInfo->new($prefix);
 
-    my $d = Parse::CPAN::Packages::Distribution->new;
-    $d->prefix($prefix);
-    $d->dist($i->dist);
-    $d->version($i->version);
-    $d->maturity($i->maturity);
-    $d->filename($i->filename);
-    $d->cpanid($i->cpanid);
-    $d->distvname($i->distvname);
- 
-    $m->distribution($d);
+      $d->prefix($prefix);
+      $d->dist($i->dist);
+      $d->version($i->version);
+      $d->maturity($i->maturity);
+      $d->filename($i->filename);
+      $d->cpanid($i->cpanid);
+      $d->distvname($i->distvname);
+      $d;
+    };
+
+    $m->distribution($dist);
+    push @{ $dist->packages }, $m;
 
     $data->{$package} = $m;
   }
@@ -70,6 +74,17 @@ sub packages {
   return values %{$self->data};
 }
 
+sub distribution {
+  my $self = shift;
+  my $dist = shift;
+  return $self->dists->{$dist};
+}
+
+sub distributions {
+  my $self = shift;
+  return values %{$self->dists};
+}
+
 1;
 
 __END__
@@ -81,17 +96,17 @@ Parse::CPAN::Packages - Parse 02packages.details.txt.gz
 =head1 SYNOPSIS
 
   use Parse::CPAN::Packages;
-  
+
   # must have downloading and un-gzip-ed
   my $p = Parse::CPAN::Packages->new("02packages.details.txt");
 
   my $m = $p->package("Acme::Colour");
   # $m is a Parse::CPAN::Packages::Package object
-  print $m->package, "\n";    # Acme::Colour
+  print $m->package, "\n";   # Acme::Colour
   print $m->version, "\n";   # 1.00
 
   my $d = $p->distribution;
-  # $d is a Parse::CPAN::Packages::Package object
+  # $d is a Parse::CPAN::Packages::Distribution object
   print $d->prefix, "\n";    # L/LB/LBROCARD/Acme-Colour-1.00.tar.gz
   print $d->dist, "\n";      # Acme-Colour
   print $d->version, "\n";   # 1.00
@@ -103,6 +118,9 @@ Parse::CPAN::Packages - Parse 02packages.details.txt.gz
   my @packages = $p->packages;
   # all the package objects
 
+  my @distributions = $p->distributions;
+  # all the distribution objects
+
 =head1 DESCRIPTION
 
 The Comprehensive Perl Archive Network (CPAN) is a very useful
